@@ -18,13 +18,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -135,6 +135,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->mime_type=ConstantString("application/postscript");
   entry->module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PS3");
   entry->encoder=(EncodeImageHandler *) WritePS3Image;
@@ -142,6 +143,7 @@ ModuleExport size_t RegisterPS3Image(void)
   entry->mime_type=ConstantString("application/postscript");
   entry->module=ConstantString("PS3");
   entry->seekable_stream=MagickTrue;
+  entry->blob_support=MagickFalse;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -258,7 +260,7 @@ static MagickBooleanType SerializeImage(const ImageInfo *image_info,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
@@ -331,7 +333,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
     bit;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
@@ -403,7 +405,7 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
     y;
 
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
@@ -463,9 +465,9 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     *pixels;
 
   assert(image_info != (ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->matte != MagickFalse);
@@ -491,38 +493,38 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     case NoCompression:
     default:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_NoCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_NoCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case FaxCompression:
     case Group4Compression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_FaxCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_FaxCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case LZWCompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_LZWCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_LZWCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case RLECompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_RLECompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_RLECompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case ZipCompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_ZipCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_ZipCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
   }
@@ -593,6 +595,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       status=SerializeImageChannel(image_info,mask_image,&pixel_info,&length);
       if (status == MagickFalse)
         break;
+      pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
       status=ZLIBEncodeImage(image,length,pixels);
       pixel_info=RelinquishVirtualMemory(pixel_info);
       break;
@@ -622,32 +625,32 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
 static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
 {
   static const char
-    *PostscriptProlog[]=
+    *const PostscriptProlog[]=
     {
       "/ByteStreamDecodeFilter",
       "{",
       "  /z exch def",
       "  /r exch def",
       "  /c exch def",
-      "  z "PS3_NoCompression" eq { /ASCII85Decode filter } if",
-      "  z "PS3_FaxCompression" eq",
+      "  z " PS3_NoCompression " eq { /ASCII85Decode filter } if",
+      "  z " PS3_FaxCompression " eq",
       "  {",
       "    <<",
-      "      /K "CCITTParam,
+      "      /K " CCITTParam,
       "      /Columns c",
       "      /Rows r",
       "    >>",
       "    /CCITTFaxDecode filter",
       "  } if",
-      "  z "PS3_JPEGCompression" eq { /DCTDecode filter } if",
-      "  z "PS3_LZWCompression" eq { /LZWDecode filter } if",
-      "  z "PS3_RLECompression" eq { /RunLengthDecode filter } if",
-      "  z "PS3_ZipCompression" eq { /FlateDecode filter } if",
+      "  z " PS3_JPEGCompression " eq { /DCTDecode filter } if",
+      "  z " PS3_LZWCompression " eq { /LZWDecode filter } if",
+      "  z " PS3_RLECompression " eq { /RunLengthDecode filter } if",
+      "  z " PS3_ZipCompression " eq { /FlateDecode filter } if",
       "} bind def",
       "",
       "/DirectClassImageDict",
       "{",
-      "  colorspace "PS3_RGBColorspace" eq",
+      "  colorspace " PS3_RGBColorspace " eq",
       "  {",
       "    /DeviceRGB setcolorspace",
       "    <<",
@@ -672,7 +675,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "      /MultipleDataSources false",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      /Decode",
-      "        compression "PS3_JPEGCompression" eq",
+      "        compression " PS3_JPEGCompression " eq",
       "        { [1 0 1 0 1 0 1 0] }",
       "        { [0 1 0 1 0 1 0 1] }",
       "        ifelse",
@@ -705,7 +708,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  {",
       "    % RGB colormap.",
       "    /colormap colors 3 mul string def",
-      "    compression "PS3_NoCompression" eq",
+      "    compression " PS3_NoCompression " eq",
       "    { currentfile /ASCII85Decode filter colormap readstring pop pop }",
       "    { currentfile colormap readstring pop pop }",
       "    ifelse",
@@ -725,7 +728,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "",
       "/NonMaskedImageDict",
       "{",
-      "  class "PS3_PseudoClass" eq",
+      "  class " PS3_PseudoClass " eq",
       "  { PseudoClassImageDict }",
       "  { DirectClassImageDict }",
       "  ifelse",
@@ -769,9 +772,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  token pop /y exch def pop",
       "  currentfile buffer readline pop",
       "  token pop /pointsize exch def pop",
-      (char *) NULL
+      (const char *) NULL
     },
-    *PostscriptEpilog[]=
+    *const PostscriptEpilog[]=
     {
       "  x y scale",
       "  % Clipping path.",
@@ -810,7 +813,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  grestore",
       "  sp { showpage } if",
       "} bind def",
-      (char *) NULL
+      (const char *) NULL
     };
 
   char
@@ -823,8 +826,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     compression;
 
   const char
+    *const *q,
     *option,
-    **q,
     *value;
 
   double
@@ -865,6 +868,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     bounds;
 
   size_t
+    imageListLength,
     length,
     page,
     pixel,
@@ -883,9 +887,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
@@ -927,9 +931,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     default:
       break;
   }
-  (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
+  (void) memset(&bounds,0,sizeof(bounds));
   page=0;
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
@@ -1587,7 +1592,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "%%%%BeginData:%13ld %s Bytes\n",(long) length,
       compression == NoCompression ? "ASCII" : "BINARY");
     (void) WriteBlobString(image,buffer);
-    offset=SeekBlob(image,stop,SEEK_SET);
+    (void) SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
     /*
       End private dictionary if this an EPS.
@@ -1598,8 +1603,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
