@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -57,6 +57,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/memory-private.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
 #include "magick/pixel-accessor.h"
@@ -240,7 +241,6 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
 
   size_t
     bytes_per_pixel,
-    lsb_first,
     max_packets,
     quantum;
 
@@ -251,6 +251,9 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
   unsigned char
     *pixels;
 
+  unsigned long
+    lsb_first;
+
   ViffInfo
     viff_info;
 
@@ -258,12 +261,12 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
+  assert(exception->signature == MagickCoreSignature);
   image=AcquireImage(image_info);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
@@ -292,6 +295,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       &viff_info.machine_dependency);
     (void) ReadBlob(image,sizeof(viff_info.reserve),viff_info.reserve);
     count=ReadBlob(image,512,(unsigned char *) viff_info.comment);
+    if (count != 512)
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
     viff_info.comment[511]='\0';
     if (strlen(viff_info.comment) > 4)
       (void) SetImageProperty(image,"comment",viff_info.comment);
@@ -303,8 +308,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     viff_info.rows=ReadBlobLong(image);
     viff_info.columns=ReadBlobLong(image);
     viff_info.subrows=ReadBlobLong(image);
-    viff_info.x_offset=(int) ReadBlobLong(image);
-    viff_info.y_offset=(int) ReadBlobLong(image);
+    viff_info.x_offset=ReadBlobSignedLong(image);
+    viff_info.y_offset=ReadBlobSignedLong(image);
     viff_info.x_bits_per_pixel=(float) ReadBlobLong(image);
     viff_info.y_bits_per_pixel=(float) ReadBlobLong(image);
     viff_info.location_type=ReadBlobLong(image);
@@ -329,6 +334,14 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
     image->rows=viff_info.columns;
     image->depth=viff_info.x_bits_per_pixel <= 8 ? 8UL :
       MAGICKCORE_QUANTUM_DEPTH;
+    image->matte=viff_info.number_data_bands == 4 ? MagickTrue : MagickFalse;
+    status=SetImageExtent(image,image->columns,image->rows);
+    if (status == MagickFalse)
+      {
+        InheritException(exception,&image->exception);
+        return(DestroyImageList(image));
+      }
+    (void) SetImageBackgroundColor(image);
     /*
       Verify that we can read this VIFF image.
     */
@@ -405,8 +418,15 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
           default: bytes_per_pixel=1; break;
         }
         image->colors=viff_info.map_columns;
+        if ((MagickSizeType) (viff_info.map_rows*image->colors) > GetBlobSize(image))
+          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
         if (AcquireImageColormap(image,image->colors) == MagickFalse)
           ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        if ((MagickSizeType) viff_info.map_rows > GetBlobSize(image))
+          ThrowReaderException(CorruptImageError,"InsufficientImageDataInFile");
+        if ((MagickSizeType) viff_info.map_rows >
+            (viff_info.map_rows*bytes_per_pixel*sizeof(*viff_colormap)))
+          ThrowReaderException(CorruptImageError,"ImproperImageHeader");
         viff_colormap=(unsigned char *) AcquireQuantumMemory(image->colors,
           viff_info.map_rows*bytes_per_pixel*sizeof(*viff_colormap));
         if (viff_colormap == (unsigned char *) NULL)
@@ -414,7 +434,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         /*
           Read VIFF raster colormap.
         */
-        count=ReadBlob(image,bytes_per_pixel*image->colors*viff_info.map_rows,
+        (void) ReadBlob(image,bytes_per_pixel*image->colors*viff_info.map_rows,
           viff_colormap);
         lsb_first=1;
         if (*(char *) &lsb_first &&
@@ -469,22 +489,18 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       default:
         ThrowReaderException(CoderError,"ColormapTypeNotSupported");
     }
-    /*
-      Initialize image structure.
-    */
-    image->matte=viff_info.number_data_bands == 4 ? MagickTrue : MagickFalse;
-    image->storage_class=
-      (viff_info.number_data_bands < 3 ? PseudoClass : DirectClass);
-    image->columns=viff_info.rows;
-    image->rows=viff_info.columns;
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    status=SetImageExtent(image,image->columns,image->rows);
-    if (status == MagickFalse)
+    if (viff_info.data_storage_type == VFF_TYP_BIT)
       {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
+        /*
+          Create bi-level colormap.
+        */
+        image->colors=2;
+        if (AcquireImageColormap(image,image->colors) == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        image->colorspace=GRAYColorspace;
       }
     /*
       Allocate VIFF pixels.
@@ -498,14 +514,26 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       default: bytes_per_pixel=1; break;
     }
     if (viff_info.data_storage_type == VFF_TYP_BIT)
-      max_packets=((image->columns+7UL) >> 3UL)*image->rows;
+      {
+        if (HeapOverflowSanityCheck((image->columns+7UL) >> 3UL,image->rows) != MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        max_packets=((image->columns+7UL) >> 3UL)*image->rows;
+      }
     else
-      max_packets=(size_t) (number_pixels*viff_info.number_data_bands);
-    pixels=(unsigned char *) AcquireQuantumMemory(max_packets,
-      bytes_per_pixel*sizeof(*pixels));
+      {
+        if (HeapOverflowSanityCheck((size_t) number_pixels,viff_info.number_data_bands) != MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        max_packets=(size_t) (number_pixels*viff_info.number_data_bands);
+      }
+    if ((MagickSizeType) (bytes_per_pixel*max_packets) > GetBlobSize(image))
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    pixels=(unsigned char *) AcquireQuantumMemory((size_t) MagickMax(
+      number_pixels,max_packets),bytes_per_pixel*sizeof(*pixels));
     if (pixels == (unsigned char *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    count=ReadBlob(image,bytes_per_pixel*max_packets,pixels);
+    (void) memset(pixels,0,MagickMax(number_pixels,max_packets)*
+      bytes_per_pixel*sizeof(*pixels));
+    (void) ReadBlob(image,bytes_per_pixel*max_packets,pixels);
     lsb_first=1;
     if (*(char *) &lsb_first &&
         ((viff_info.machine_dependency != VFF_DEP_DECORDER) &&
@@ -596,7 +624,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
             if (value < 0)
               value=0;
         }
-      *p=(unsigned char) value;
+      *p=(unsigned char) ((Quantum) value);
       p++;
     }
     /*
@@ -696,13 +724,13 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
 
                   index=(ssize_t) GetPixelRed(q);
                   SetPixelRed(q,image->colormap[(ssize_t)
-                    ConstrainColormapIndex(image,index)].red);
+                    ConstrainColormapIndex(image,(ssize_t) index)].red);
                   index=(ssize_t) GetPixelGreen(q);
                   SetPixelGreen(q,image->colormap[(ssize_t)
-                    ConstrainColormapIndex(image,index)].green);
+                    ConstrainColormapIndex(image,(ssize_t) index)].green);
                   index=(ssize_t) GetPixelRed(q);
                   SetPixelBlue(q,image->colormap[(ssize_t)
-                    ConstrainColormapIndex(image,index)].blue);
+                    ConstrainColormapIndex(image,(ssize_t) index)].blue);
                 }
               SetPixelOpacity(q,image->matte != MagickFalse ? QuantumRange-
                 ScaleCharToQuantum(*(p+number_pixels*3)) : OpaqueOpacity);
@@ -736,7 +764,7 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     count=ReadBlob(image,1,&viff_info.identifier);
-    if ((count != 0) && (viff_info.identifier == 0xab))
+    if ((count == 1) && (viff_info.identifier == 0xab))
       {
         /*
           Allocate next image structure.
@@ -744,8 +772,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
         AcquireNextImage(image_info,image);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
-            image=DestroyImageList(image);
-            return((Image *) NULL);
+            status=MagickFalse;
+            break;
           }
         image=SyncNextImageInList(image);
         status=SetImageProgress(image,LoadImagesTag,TellBlob(image),
@@ -755,6 +783,8 @@ static Image *ReadVIFFImage(const ImageInfo *image_info,
       }
   } while ((count != 0) && (viff_info.identifier == 0xab));
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -790,12 +820,14 @@ ModuleExport size_t RegisterVIFFImage(void)
   entry->decoder=(DecodeImageHandler *) ReadVIFFImage;
   entry->encoder=(EncodeImageHandler *) WriteVIFFImage;
   entry->magick=(IsImageFormatHandler *) IsVIFF;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Khoros Visualization image");
   entry->module=ConstantString("VIFF");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("XV");
   entry->decoder=(DecodeImageHandler *) ReadVIFFImage;
   entry->encoder=(EncodeImageHandler *) WriteVIFFImage;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Khoros Visualization image");
   entry->module=ConstantString("VIFF");
   (void) RegisterMagickInfo(entry);
@@ -937,6 +969,9 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
   register unsigned char
     *q;
 
+  size_t
+    imageListLength;
+
   ssize_t
     y;
 
@@ -950,16 +985,17 @@ static MagickBooleanType WriteVIFFImage(const ImageInfo *image_info,
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) ResetMagickMemory(&viff_info,0,sizeof(ViffInfo));
+  (void) memset(&viff_info,0,sizeof(ViffInfo));
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
@@ -1170,7 +1206,6 @@ RestoreMSCWarning
             /*
               Convert PseudoClass image to a VIFF monochrome image.
             */
-            (void) SetImageType(image,BilevelType);
             for (y=0; y < (ssize_t) image->rows; y++)
             {
               p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
@@ -1232,8 +1267,7 @@ RestoreMSCWarning
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
