@@ -17,13 +17,13 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2015 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -62,6 +62,7 @@
 #include "magick/string_.h"
 #include "magick/token.h"
 #include "magick/utility.h"
+#include "magick/utility-private.h"
 #if defined(MAGICKCORE_MODULES_SUPPORT)
 #if defined(MAGICKCORE_LTDL_DELEGATE)
 #include "ltdl.h"
@@ -141,13 +142,13 @@ MagickExport ModuleInfo *AcquireModuleInfo(const char *path,const char *tag)
   module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
   if (module_info == (ModuleInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
-  (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
+  (void) memset(module_info,0,sizeof(*module_info));
   if (path != (const char *) NULL)
     module_info->path=ConstantString(path);
   if (tag != (const char *) NULL)
     module_info->tag=ConstantString(tag);
   module_info->timestamp=time(0);
-  module_info->signature=MagickSignature;
+  module_info->signature=MagickCoreSignature;
   return(module_info);
 }
 
@@ -381,22 +382,11 @@ static int ModuleCompare(const void *x,const void *y)
 }
 #endif
 
-static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
-  struct dirent **result)
-{
-#if defined(MAGICKCORE_HAVE_READDIR_R)
-  return(readdir_r(directory,entry,result));
-#else
-  (void) entry;
-  errno=0;
-  *result=readdir(directory);
-  return(errno);
-#endif
-}
-
 MagickExport char **GetModuleList(const char *pattern,
   const MagickModuleType type,size_t *number_modules,ExceptionInfo *exception)
 {
+#define MaxModules  511
+
   char
     **modules,
     filename[MaxTextExtent],
@@ -443,7 +433,7 @@ MagickExport char **GetModuleList(const char *pattern,
   if (status == MagickFalse)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
-  max_entries=255;
+  max_entries=MaxModules;
   modules=(char **) AcquireQuantumMemory((size_t) max_entries+1UL,
     sizeof(*modules));
   if (modules == (char **) NULL)
@@ -544,6 +534,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
+  if (strchr(filename,'/') != (char *) NULL)
+    return(MagickFalse);
   (void) CopyMagickString(path,filename,MaxTextExtent);
   module_path=(char *) NULL;
   switch (module_type)
@@ -588,6 +580,15 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         if ((q >= path) && (*q != *DirectorySeparator))
           (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
         (void) ConcatenateMagickString(path,filename,MaxTextExtent);
+#if defined(MAGICKCORE_HAVE_REALPATH)
+        {
+          char
+            resolved_path[PATH_MAX+1];
+
+          if (realpath(path,resolved_path) != (char *) NULL)
+            (void) CopyMagickString(path,resolved_path,MaxTextExtent);
+        }
+#endif
         if (IsPathAccessible(path) != MagickFalse)
           {
             module_path=DestroyString(module_path);
@@ -776,9 +777,25 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     char
       *home;
 
-    home=GetEnvironmentValue("HOME");
+    home=GetEnvironmentValue("XDG_CONFIG_HOME");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("LOCALAPPDATA");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("APPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("USERPROFILE");
+    if (home != (char *) NULL)
+      {
+        /*
+          Search $XDG_CONFIG_HOME/ImageMagick.
+        */
+        (void) FormatLocaleString(path,MaxTextExtent,"%s%sImageMagick%s%s",
+          home,DirectorySeparator,DirectorySeparator,filename);
+        home=DestroyString(home);
+        if (IsPathAccessible(path) != MagickFalse)
+          return(MagickTrue);
+      }
+    home=GetEnvironmentValue("HOME");
     if (home != (char *) NULL)
       {
         /*
@@ -876,20 +893,24 @@ static MagickBooleanType IsModuleTreeInstantiated(
           ModuleInfo
             *module_info;
 
-          module_list=NewSplayTree(CompareSplayTreeString,
+          SplayTreeInfo
+            *splay_tree;
+
+          splay_tree=NewSplayTree(CompareSplayTreeString,
             (void *(*)(void *)) NULL,DestroyModuleNode);
-          if (module_list == (SplayTreeInfo *) NULL)
+          if (splay_tree == (SplayTreeInfo *) NULL)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
           module_info=AcquireModuleInfo((const char *) NULL,"[boot-strap]");
           module_info->stealth=MagickTrue;
-          status=AddValueToSplayTree(module_list,module_info->tag,module_info);
+          status=AddValueToSplayTree(splay_tree,module_info->tag,module_info);
           if (status == MagickFalse)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
           if (lt_dlinit() != 0)
             ThrowFatalException(ModuleFatalError,
               "UnableToInitializeModuleLoader");
+          module_list=splay_tree;
         }
       UnlockSemaphoreInfo(module_semaphore);
     }
@@ -957,10 +978,18 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
     Find the module.
   */
   assert(images != (Image **) NULL);
-  assert((*images)->signature == MagickSignature);
+  assert((*images)->signature == MagickCoreSignature);
   if ((*images)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       (*images)->filename);
+  rights=ReadPolicyRights;
+  if (IsRightsAuthorized(FilterPolicyDomain,rights,tag) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",tag);
+      return(MagickFalse);
+    }
 #if !defined(MAGICKCORE_BUILD_MODULES)
   {
     MagickBooleanType
@@ -971,14 +1000,6 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
       return(status);
   }
 #endif
-  rights=ReadPolicyRights;
-  if (IsRightsAuthorized(FilterPolicyDomain,rights,tag) == MagickFalse)
-    {
-      errno=EPERM;
-      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
-        "NotAuthorized","`%s'",tag);
-      return(MagickFalse);
-    }
   TagToFilterModuleName(tag,name);
   status=GetMagickModulePath(name,MagickImageFilterModule,path,exception);
   if (status == MagickFalse)
@@ -1004,7 +1025,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
   (void) FormatLocaleString(name,MaxTextExtent,"%sImage",tag);
 #else
   (void) FormatLocaleString(name,MaxTextExtent,"%s%sImage",
-    MAGICKCORE_NAMESPACE_PREFIX,tag);
+    MAGICKCORE_NAMESPACE_PREFIX_TAG,tag);
 #endif
   /*
     Execute the module.
@@ -1241,6 +1262,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   ModuleInfo
     *module_info;
 
+  PolicyRights
+    rights;
+
   register const CoderInfo
     *p;
 
@@ -1254,6 +1278,14 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
+  rights=ReadPolicyRights;
+  if (IsRightsAuthorized(ModulePolicyDomain,rights,module) == MagickFalse)
+    {
+      errno=EPERM;
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",module);
+      return(MagickFalse);
+    }
   (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
@@ -1368,8 +1400,12 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   (void) GetMagickInfo((char *) NULL,exception);
   number_modules=0;
   modules=GetModuleList("*",MagickImageCoderModule,&number_modules,exception);
-  if (modules == (char **) NULL)
-    return(MagickFalse);
+  if ((modules == (char **) NULL) || (*modules == (char *) NULL))
+    {
+      if (modules != (char **) NULL)
+        modules=(char **) RelinquishMagickMemory(modules);
+      return(MagickFalse);
+    }
   for (i=0; i < (ssize_t) number_modules; i++)
     (void) OpenModule(modules[i],exception);
   /*
@@ -1416,7 +1452,7 @@ static const ModuleInfo *RegisterModule(const ModuleInfo *module_info,
     status;
 
   assert(module_info != (ModuleInfo *) NULL);
-  assert(module_info->signature == MagickSignature);
+  assert(module_info->signature == MagickCoreSignature);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",module_info->tag);
   if (module_list == (SplayTreeInfo *) NULL)
     return((const ModuleInfo *) NULL);
@@ -1560,7 +1596,7 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
       prefix_format[MaxTextExtent];
 
     (void) FormatLocaleString(prefix_format,MaxTextExtent,"%s%s",
-      MAGICKCORE_NAMESPACE_PREFIX,format);
+      MAGICKCORE_NAMESPACE_PREFIX_TAG,format);
     (void) FormatLocaleString(module,MaxTextExtent,prefix_format,name);
   }
 #endif
@@ -1634,7 +1670,7 @@ MagickExport MagickBooleanType InvokeDynamicImageFilter(const char *tag,
     rights;
 
   assert(image != (Image **) NULL);
-  assert((*image)->signature == MagickSignature);
+  assert((*image)->signature == MagickCoreSignature);
   if ((*image)->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
   rights=ReadPolicyRights;
